BEGIN {
  FS="[ <>-]+";
  target_region = 0;
  comment = 0;
  charset = 0;
}

# Check only the section Appendix.

/^Appendix/ {
  target_region = 1;
}

/^A.8./ {
  target_region = 0;
}

{
  if (! target_region) continue;
}

# Skip header lines.

/^Zhu, et al/ { continue; }
/^RFC/ { continue; }
/^$/ { continue; }
/^/ { continue; }

# Lines of the format something linke "0xA140-0xA1F5 <-> 0x2121-0x2256".

/[ \t]+[0-9A-Zx]+[- \t]+[0-9A-Zx]+ <-> [0-9A-Zx]+[- \t]+[ \t]*[0-9A-Zx]+/ {
  if (comment) { print " */"; comment = 0; }
  printf $2", "$3", "$4", "charset",";
  if (NF <= 5) {
    print "";
  } else {
    print "\t/* "$6, $7, $8, $9" */";
  }
  continue;
}

# Lines of the format something linke "0xA1F6 <-> 0x2258".

/[ \t]+[0-9A-Zx]+ <-> [0-9A-Zx]+/ {
  if (comment) { print " */"; comment = 0; }
  temp_charset = charset;
  if ($0 ~ "duplicate of Level 1") temp_charset = "CHARSET_CNS11643_1";
  printf $2", "$2", "$3", "temp_charset",";
  if (NF <= 3) {
    print "";
  } else {
    print "\t/* "$4, $5, $6, $7, $8, $9" */";
  }
  continue;
}

# Subsections change a target character set.

/^A.1./ {
  print " */";
  comment = 0;
  print "unsigned short BtoCNS[] = {";
  charset = "CHARSET_CNS11643_1";
}
/^A.2./ { charset = "CHARSET_CNS11643_1"; }
/^A.3./ { charset = "CHARSET_CNS11643_2"; }
/^A.4./ { charset = "CHARSET_CNS11643_1"; }
/^A.5./ { charset = "CHARSET_CNS11643_1"; }
/^A.6./ { charset = "CHARSET_CNS11643_3"; }
/^A.7./ { charset = "CHARSET_CNS11643_4"; }

# Any other lines are inserted as comments.

{
  if (! comment) {
    printf "/* ";
    comment = 1;
  } else {
    printf "   ";
  }
  print $0;
}

END {
  if (comment) print " */";
  print "}; ";
}
