/* b5map.c -- Mapping between Big5 and CNS11643/GB2312

   Copyright (C) 1995, 1996 Electrotechnical Laboratory, JAPAN.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */


/* Author: K.Handa <handa@etl.go.jp>
   History:
	1996.9.10 -- Version 0.9 beta released.
*/

#include <stdio.h>
#include "iso2022.h"

/* File `b5cns.h' is generated by the awk program `b5cns.awk' from the
   text file `RFC1922.txt'.  This file contains mapping information
   between Big5 and CNS11643 Plane 1,2,3, and 4.  */
#include "b5cns.h"

/* File `b2g_tables.S.c' is extracted from:
	software: BeTTY/CCF/B5Encode 3-in-1 package
	version : 1.534, Mar. 23, 1995.
   This file cotains mapping information between GB2312 and Big5.  */
#include "b2g_tables.S.c"

#ifndef HAVE_BZERO
#define bzero(b, len) memset (b, 0, len)
#endif

unsigned short b5_cns[BIG5_PACK_MAX][2];
unsigned short b5_gb[BIG5_PACK_MAX];
unsigned short iso_b5[5][ISO_PACK_MAX];

void
b5map_init ()
{
  int i, len;
  register unsigned int charset, charset_idx, high, low;

  bzero (b5_cns, sizeof b5_cns);
  bzero (b5_gb, sizeof b5_gb);
  bzero (iso_b5, sizeof iso_b5);

  len = (sizeof BtoCNS) / (sizeof BtoCNS[0]);
  for (i = 0; i < len; i += 4)
    {
      int from, to, j, b5, iso;

      b5 = BtoCNS[i];
      high = b5 >> 8, low = b5 & 0xFF;
      from = BIG5_PACK (high, low);
      b5 = BtoCNS[i + 1];
      high = b5 >> 8, low = b5 & 0xFF;
      to = BIG5_PACK (high, low);
      charset = BtoCNS[i + 3];
      charset_idx = charset - CHARSET_CNS11643_1 + 1;
      iso = ISO_PACK (BtoCNS[i + 2]);
      for (j = from; j <= to; j++, iso++)
	{
	  b5_cns[j][0] = charset;
	  b5_cns[j][1] = ISO_UNPACK (iso);
	  BIG5_UNPACK (j, high, low);
	  iso_b5[charset_idx][iso] = (high << 8) | low;
	}
    }

  len = (sizeof GtoB) / (sizeof GtoB[0]);
  for (i = 0; i < len; i += 2)
    {
      unsigned int gb = (i < 1692 ? i : i + 2820 - 1692) / 2, b5;

      high = GtoB[i], low = GtoB[i + 1];
      b5 = BIG5_PACK (high, low);
      b5_gb[b5] = ISO_UNPACK (gb);
      iso_b5[0][gb] = (high << 8) | low;
    }
}
