/* iso2022.h  -- Header for ISO-2022-CN (RFC1922) encoder/decoder
   Copyright (C) 1995, 1996 Electrotechnical Laboratory, JAPAN.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#define ISO_SO	0x0E		/* shift-out */
#define ISO_SI	0x0F		/* shift-in */
#define ISO_ESC	0x1B		/* escape */
#define ISO_SS2_7 'N'		/* single shift 2, 7-bit code */
#define ISO_SS3_7 'O'		/* single shift 3, 7-bit code */
#define ISO_SS2_8 0x8E		/* single shift 2, 8-bit code */
#define ISO_SS3_8 0x8F		/* single shift 3, 8-bit code */


#define CHARSET_ASCII		0
#define CHARSET_GB2312		'A'
#define CHARSET_GB7589		'0'
#define CHARSET_GB7590		'1'
#define CHARSET_GB12345		'2'
#define CHARSET_GB13131		'3'
#define CHARSET_GB13132		'4'
#define CHARSET_CNS11643_1	'G'
#define CHARSET_CNS11643_2	'H'
#define CHARSET_CNS11643_3	'I'
#define CHARSET_CNS11643_4	'J'
#define CHARSET_CNS11643_5	'K'
#define CHARSET_CNS11643_6	'L'
#define CHARSET_CNS11643_7	'M'
#define CHARSET_ISOIR165	'E'

#define BLANK_SQUARE_GB2312	0x2175
#define BLANK_SQUARE_CNS11643_1	0x217B
#define BLANK_SQUARE_BIG5	0xA1BC

/* For mapping between Big5 and ISO-2022-CN.  */
/* Big5 (1st byte)      0xA1 .. 0xFE
        (2nd byte)      0x40 .. 0x7E and 0xA1 .. 0xFE */

/* I hope all constant calculations be resolved by a compiler. */

#define BIG5_ROW (0xFF - 0xA1 + 0x7F - 0x40)
#define BIG5_OFF_1 0x40
#define BIG5_OFF_2 (0xA1 - 0x7F + 0x40)

#define BIG5_PACK_1(bh, bl) \
  (((bh) - 0xA1) * BIG5_ROW + (bl) - BIG5_OFF_1)
#define BIG5_PACK_2(bh, bl) \
  (((bh) - 0xA1) * BIG5_ROW + (bl) - BIG5_OFF_2)
#define BIG5_PACK(bh, bl) \
  ((bl) < 0x7F ? BIG5_PACK_1 (bh, bl) : BIG5_PACK_2 (bh, bl))
#define BIG5_UNPACK(b, bh, bl) \
  (bh = (((b) / BIG5_ROW) + 0xA1), \
   bl = (b) % BIG5_ROW, \
   bl += (bl < (0x7F - 0x40)) ? BIG5_OFF_1 : BIG5_OFF_2)
#define BIG5_PACK_MAX BIG5_PACK_2 (0xFE, 0xFE)

#define ISO_ROW (0x7F - 0x21)
#define ISO_OFF 0x21

#define ISO_PACK(c) \
  ((((c) >> 8) - ISO_OFF) * ISO_ROW + ((c) & 0xFF) - ISO_OFF)
#define ISO_UNPACK(c) \
  ((((c) / ISO_ROW + ISO_OFF) << 8) | ((c) % ISO_ROW + ISO_OFF))
#define ISO_PACK_MAX ISO_PACK (0xFEFE)

extern unsigned short b5_cns[BIG5_PACK_MAX][2];
extern unsigned short b5_gb[BIG5_PACK_MAX];
extern unsigned short iso_b5[5][ISO_PACK_MAX];

#define BIG5_TO_ISO(b1, b2, charset, iso)		\
  do {							\
    unsigned short packed_b5 = BIG5_PACK (b1, b2);	\
    if (charset = b5_cns[packed_b5][0])			\
      iso = b5_cns[packed_b5][1];			\
    else if (iso = b5_gb[packed_b5])			\
      charset = CHARSET_GB2312;				\
  } while (0)

#define BIG5_TO_CNS(b1, b2, charset, cns)		\
  do {							\
    unsigned short packed_b5 = BIG5_PACK (b1, b2);	\
    if (charset = b5_cns[packed_b5][0])			\
      cns = b5_cns[packed_b5][1];			\
  } while (0)

#define BIG5_TO_GB(b1, b2) b5_gb[BIG5_PACK (b1, b2)]

#define ISO_TO_BIG5(charset, i1, i2)					\
  iso_b5								\
    [charset == CHARSET_GB2312 ? 0 : charset - CHARSET_CNS11643_1 + 1]	\
    [ISO_PACK (((i1) << 8) | (i2))]

/* Enable de2022cn to be used interactively at the cost of conversion
   speed.  */
#define INTERACTIVE
